# -*- coding: utf-8 -*-
# Copyright 2020 Spotify AB

__author__ = "sigint"
__version__ = "21.9.0.dev1"
__email__ = "sigint@spotify.com"
__description__ = "Internal plugin for klio"
__uri__ = "https://ghe.spotify.net/sigint/spotify-klio"
